
#ifndef __KERNEL__
 #define __KERNEL__
#endif

#ifndef MODULE
 #define MODULE
#endif

#include <linux/config.h>
#include <linux/module.h>
#include <linux/fs.h>
#include <linux/fcntl.h>
#include <linux/ioctl.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/proc_fs.h>
#include <asm/system.h>
#include <asm/segment.h>
#include <asm/io.h>
#include <linux/ioport.h>
#include <linux/init.h>
#include <linux/pci.h>

#include "dwatchdog.h"
#include "sysdep.h"   

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,2,0)
#include <asm/uaccess.h>
#endif

/* I defined some macro for device_open function */
#define TYPE(dev) (MINOR(dev) >> 4) 
#define NUM(dev) (MINOR(dev) & 0x0f)

#ifndef KERNEL_VERSION
#define KERNEL_VERSION(a,b,c) ((a)*65536+(b)*256+(c))
#endif

#ifndef DECISION_VENDOR_ID
#define DECISION_VENDER_ID 0x6666
#endif

#ifndef DECISION_DEVICE_ID
#define DECISION_DEVICE_ID 0x4000
#endif

int Base_Addr;

/*---------------------------------------------------------------------*/
int dwatchdog_open(struct inode *inoder,struct file *flip)
{
/* int type= TYPE(inoder->i_rdev);
 int num=NUM(inoder->i_rdev);
*/
 MOD_INC_USE_COUNT;
 return 0;
}
/*---------------------------------------------------------------------*/
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,2,0)
static int dwatchdog_release(struct inode *inoder,struct file *flip)
#else
static void dwatchdog_release(struct inode *inoder,struct file *flip)
#endif
{
  MOD_DEC_USE_COUNT;
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,2,0)
  return 0;
#endif    
}
/*---------------------------------------------------------------------*/
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,2,0)
static ssize_t dwatchdog_read(struct file *flip,char *buf,size_t count,loff_t *ptr)
#else
static int dwatchdog_read(struct inode *inode,struct file *flip, char *buf,count_t count)
#endif
{
 int countt=0;
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,2,0)
  if(ptr!=&flip->f_pos)  /* can't seek on this device */
  return -ESPIPE;
#endif  
switch(MINOR(flip->f_dentry->d_inode->i_rdev))
 {
   case dwatchdog_MINOR: 	 
           buf[0]=readb(Base_Addr+4);
           buf[1]=readb(Base_Addr+5);
           buf[2]=readb(Base_Addr+6);
           countt=3;
           break;
   default:
           return -EINVAL;        
 }          
 return countt;
}
/*---------------------------------------------------------------------*/
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,2,0)
static ssize_t dwatchdog_write(struct file *flip,const char *buf,size_t count,loff_t *ptr)
#else
static int dwatchdog_write(struct inde *inode,struct file *flip,char *buf,count_t count)
#endif
{
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,2,0)	
 if(ptr!=&flip->f_pos)  /* can't seek on this device */
  return -ESPIPE;
#endif
 
 return 0;       /* do nothing */
}
/*---------------------------------------------------------------------*/
int Myread(char *buf,int count)
{
 int countt;
 switch(buf[0])
  {
   case 0:
           buf[1]=inb(Base_Addr);
           countt=2;
           break;
   case 1:
           buf[1]=inb(Base_Addr+1);
           countt=2;
           break;
   case 2:
           buf[1]=inb(Base_Addr+4);
           buf[2]=inb(Base_Addr+5);
           buf[3]=inb(Base_Addr+6);
           countt=4;
           break;
   case 3:
   case 4:
   case 5:
           buf[1]=inb(Base_Addr+7);
           countt=2;
           break;
   default:
           countt=0;
           break;
  }
 return countt;

	
}
/*---------------------------------------------------------------------*/
void Mywrite(char *buf,int count)
{
 char ch;
 switch(buf[0])
  {
   case 0:
           outb(0,Base_Addr);
           break;
   case 1:
           outb(0,Base_Addr+1);
           break;
   case 2:
           outb(buf[1],Base_Addr+4);
           outb(buf[2],Base_Addr+5);
           outb(buf[3],Base_Addr+6);
           break;
   case 3:
           ch=buf[1];
           ch=(char)(ch|buf[2]);
           outb(ch,Base_Addr+7);
           break;
   case 4:
           ch=buf[1];
           ch=buf[2]==0 ? ch|0x10 : ch&0xef;
           outb(ch,Base_Addr+7);
           break;
   case 5:
           ch=buf[1];
           ch=ch|0x20;
           outb(ch,Base_Addr+7);
           break;
   default:
           
           break;

  }
}
/*---------------------------------------------------------------------*/
int dwatchdog_ioctl(
    struct inode *inode,
    struct file *file,
    unsigned int ioctl_num,/* The number of the ioctl */
    unsigned long ioctl_param) /* The parameter to it */
{
  char param[4],param1[4];
  unsigned long int ch1=0;
  char *temp=(char *)ioctl_param;
  int i;
  char ch=0;
  switch (ioctl_num) 
   {
    case DECISION_ENABLE_COUNTER:
      param[0]=0;
      Mywrite(param ,1);
      break;

    case DECISION_DISABLE_COUNTER:
      param[0]=1;
      Mywrite(param , 1);
      break;

    case DECISION_GET_COUNTER:
      param[0]=2;
      Myread(param,4);
      param1[0]=param[1];
      param1[1]=param[2];
      param1[2]=(char)(param[3]&0x0f);
      param1[3]=0;
      for(i=0;i<sizeof(unsigned long int);i++,temp++)
       put_user(param1[i],temp);      
      break; 
      
    case DECISION_SET_COUNTER:
      ch1=(unsigned long int)ioctl_param;
      param[0]=2;
      param[1]=(char) ch1 & 0xff;
      param[2]=(char)( (ch1 & 0x00ff00)>>8) ;
      param[3]=(char)( (ch1 & 0xff0000)>>16);
      Mywrite(param , 4);
      break;
      
    case DECISION_SET_COUNTER_SOURCE:
      i=(int)temp;
      if(i>255)
       {
      #if LINUX_VERSION_CODE < KERNEL_VERSION(2,2,0)
       ch=get_user(temp);
      #else
       get_user(ch,temp);
      #endif
        }
        else
        ch=(char)i;
      param[0]=3;      
      Myread(param,2);
      param[2]= (char)(ch&0x0f);
      Mywrite(param , 3);
      break;

    case DECISION_SET_REMOTEPOWER_ON:
      param[0]=4;
      i=(int)temp;
      Myread(param , 2);
      if(i>255)
       {
      #if LINUX_VERSION_CODE < KERNEL_VERSION(2,2,0)
       ch=get_user(temp);
      #else
       get_user(ch,temp);
      #endif
       }
      else
       ch=(char)i; 
      param[2]= ch !=0 ? 1 : 0;
      Mywrite(param , 3);
      break;

    case DECISION_ENABLE_PCI_RESET:
      param[0]=5;
      Myread(param , 2);
      Mywrite(param , 2);
      break;

  }

  return 0;
}
/*---------------------------------------------------------------------*/
/*---------------------------------------------------------------------*/
static struct file_operations dwatchdog_fops={
NULL,  /* dwatchdog_lseek, */           
&dwatchdog_read,
&dwatchdog_write,
NULL,  /* dwatchdog_readdir */
NULL,  /* dwatchdog_select */
&dwatchdog_ioctl ,
NULL,  /* dwatchdog_mmap */
&dwatchdog_open,
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,2,0)
NULL,   /*  flush*/
#endif
&dwatchdog_release,
};
/*---------------------------------------------------------------------*/
/*---------------------------------------------------------------------*/
/*---------------------------------------------------------------------*/
/*---------------------------------------------------------------------*/
MODULE_AUTHOR("Delta");
MODULE_DESCRIPTION("Decision WATCH DOG Driver");
/*---------------------------------------------------------------------*/
/*---------------------------------------------------------------------*/

int init_module(void)
{
 int index=0;
 struct pci_dev *dev = NULL;
 static int dev_count=0;
 dev_count++;
 if(dev_count>1)
  {
   printk(KERN_ERR"This drive only can open one time.\n");
   return -ENODEV;
  }

#ifdef CONFIG_PCI
 if (!pci_present())   /* No PCI bus in this machine! */
#endif
 return -ENODEV;

 while (index < 1 && dev == NULL && 
        (dev = pci_find_device(DECISION_VENDER_ID, DECISION_DEVICE_ID,dev))) 
 {
  index++;
 }
  if(dev==NULL)
  return -ENODEV;
/*
  for(i=0;i<6;i++)
   {
    printk("Watch Dog Base[%d] : %x\n",i,(int)(dev->base_address[i] & PCI_BASE_ADDRESS_IO_MASK));
   } 
 */   
  Base_Addr=dev->base_address[3] & PCI_BASE_ADDRESS_IO_MASK;   
 printk("Watch Dog:I got a base %x\n",Base_Addr); 
 if(check_region(Base_Addr,8)>=0)
  request_region(Base_Addr,8,DEVICE_NAME);
 index=register_chrdev(dwatchdog_major,DEVICE_NAME,&dwatchdog_fops);
 if(index<0)
  {
  release_region(Base_Addr,8);
  return index;
  }
 printk("Watch Dog:Register Successful\n");  
 return 0;

}
/*---------------------------------------------------------------------*/
void cleanup_module(void)
{
  release_region(Base_Addr,8); 
  unregister_chrdev(dwatchdog_major,DEVICE_NAME);
 printk("WatchDog :Unregister Successful!!\n");  
}
/*---------------------------------------------------------------------*/
